#
# This script is responsible for populating the ISE project database.
#

proc Release { _iInterface } {
   if { $_iInterface == 0 } {
      return 0
   }

   # release the project
   set iRefCount [$_iInterface GetInterface $::xilinx::Prjrep::IRefCountID]
   if { $iRefCount == 0 } {
      return 1
   }
   set RESULT [$iRefCount Release]
   if { 1 != $RESULT } {
      return 0
   }
   return 1
}

proc SetDesignPlatformProperty { iProjectHelper prop_name prop_val } {

   set m_iProjectHelper $iProjectHelper

   set strvalue ""
   set return_val 1
            
   $m_iProjectHelper GetDesignPlatform iunk
   set iDesignPlatform [ $iunk GetInterface $xilinx::Dpm::IDesignPlatformID ]

   $iDesignPlatform GetPropertyInstances plat_props 

   for { $plat_props First } { ![ $plat_props IsEnd ] } { $plat_props Next }  {
      set iprop [ $plat_props CurrentItem ]
      set val ""
      set key ""
      $iprop GetStringValue val
      $iprop GetName key
      if { $key == $prop_name } {
         set return_val [ $iprop SetStringValue $prop_val ]

         break
      }
   }

   $iDesignPlatform UpdateDesignPlatform 

   Release $plat_props
   Release $iDesignPlatform

   if {$return_val == 0 } {
      return 0
   }
   return 1
}

proc SetProperty { iProjectHelper internal_name value { context 0 } } { 

   # get the dpm project
   set iunk 0
   $iProjectHelper GetDpmProject iunk
   set iProject [ $iunk GetInterface $xilinx::Dpm::IProjectID ]

   # get the design
   set iDesign 0
   $iProject GetDesign iDesign

   # get the property manager from the dpm project
   set iPropManager 0
   $iDesign GetPropertyManager iPropManager

   # get the property store
   set iDataStore 0
   $iDesign GetDataStore iDataStore

   if { $context != 0 } {
      set iContext [ $context GetInterface $xilinx::Dpm::IPropertyContextID ]
   } else {
      set iContext 0
   }


   # get the property component from datastore
   $iDataStore GetComponentByName $internal_name iInterface_PropComp

   # get the property template from the prop component
   set iPropTemplate [$iInterface_PropComp GetInterface $::xilinx::Dpm::IPropertyTemplateID]

   # get the property instance from property manager...
   set iPropInst 0
   set retVal [$iPropManager GetPropertyInstance $iPropTemplate $iContext iPropInst]

   # ... and set it.
   if { $iPropInst != 0 } {
      set retVal [ $iPropInst SetStringValue $value ]
   }


   Release $iPropInst
   Release $iDataStore
   Release $iPropManager
   Release $iDesign
   Release $iProject
}


proc main { project_file } {
   # load the libraries
   set libExt [info sharedlibextension]
   load "libCit_CoreStub$libExt"
   load "libDpm_PnfStub$libExt"
   load "libDpm_DesignDataStub$libExt"
   load "libDpm_DefnDataStub$libExt"
   load "libDpm_SupportStub$libExt"
   load "libPrjrep_CommonStub$libExt"

   # create a project helper
   set ProjectHelperID "{0725c3d2-5e9b-4383-a7b6-a80c932eac21}"
   set iunk [ Xilinx::Cit::FactoryCreate $ProjectHelperID ]
   if { $iunk == 0 } {
      return -code error "Could not create with $compid\n"
   }
   set iProjectHelper [ $iunk GetInterface $::xilinx::Dpm::IProjectHelperID ]
   if { $iProjectHelper == 0 } {
      return -code error "Interface not found.  Compid $compid, interface id $interfaceid\n"
   }

   # open the project
   set res [ $iProjectHelper Open $project_file ]

   # try and open it a second time if the first time failed.
   # this is due to a lock issue.
   if { $res != 1 } {
      set res [ $iProjectHelper Open $project_file ]
   }

   #
   # Set global properties
   #
   SetTopProps $iProjectHelper

   # call init
   $iProjectHelper Init

   # get a source helper
   set iSrcHelper [ $iProjectHelper GetInterface $::xilinx::Dpm::IProjSrcHelperID ]

   # Set the optional report component to NULL (we don't care about the
   # information now when automatically building up the project)
   set report_comp 0

   #
   # Add source files
   #
   AddSources $iSrcHelper $iProjectHelper $report_comp

   #
   # Set synthesizer/simulator specific properties
   #
   SetCustomProps $iProjectHelper

   #
   # Set the top design unit property on the synthesis view.
   #
   # First, get the synthesis view for use as a context
   set SynthesisOnly 4
   set iViewNetworkHelp [ $iProjectHelper GetInterface $::xilinx::Dpm::IViewNetworkHelpID ]
   set synth_view 0
   $iViewNetworkHelp GetDefaultViewNetwork $SynthesisOnly synth_view
   SetTop $iProjectHelper $synth_view

   # close the project file
   $iProjectHelper Close

   # let go of the component (delete it)
   Release $iProjectHelper

   #Add Map and Par Options using Customer Viewable TCL
   project open $project_file

   #Obtain the device family targetted
   set family [ project get "family" ]

   #Mapper options 
   if { $family == "Virtex5" } {
     project set "Placer Effort Level" High -process "Map"
   } else {
     project set "Perform Timing-Driven Packing and Placement" TRUE -process "Map" 
     project set "Map Effort Level" High -process "Map"
   }
   project set "Register Duplication" TRUE -process "Map" 

   #PAR options
   project set "Place & Route Effort Level (Overall)" High -process "Place & Route"

   project close
   
}
#
# Created by System Generator     Mon Feb 16 14:42:34 2015
#
# Note: This file is produced automatically, and will be overwritten the next
# time you press "Generate" in System Generator.
#


proc SetTopProps {iProjectHelper} {
    # set the project properties
    SetDesignPlatformProperty $iProjectHelper "PROP_DevFamily" "Virtex2P"
    SetDesignPlatformProperty $iProjectHelper "PROP_DevDevice" "xc2vp30"
    SetDesignPlatformProperty $iProjectHelper "PROP_DevPackage" "ff896"
    SetDesignPlatformProperty $iProjectHelper "PROP_DevSpeed"   "-7"
    SetDesignPlatformProperty $iProjectHelper "PROP_Synthesis_Tool" "XST (VHDL/Verilog)"
    SetDesignPlatformProperty $iProjectHelper "PROP_Simulator" "Modelsim-SE VHDL"
}

proc AddSources {iSrcHelper iProjectHelper report_comp} {
    # Add source files
    $iSrcHelper AddSourceFile "edgepenh_cw.vhd" report_comp
    $iSrcHelper AddSourceFile "edgepenh.vhd" report_comp
}

proc SetCustomProps {iProjectHelper} {
    # Synthesizer Properties
    SetProperty $iProjectHelper "PROP_xstUseSynthConstFile" true
    SetProperty $iProjectHelper "PROP_SynthConstraintsFile" "edgepenh_cw.xcf"
    SetProperty $iProjectHelper "PROP_xstBusDelimiter" ()
    SetProperty $iProjectHelper "PROP_xstWriteTimingConstraints" true
    SetProperty $iProjectHelper "PROPEXT_xilxSynthMaxFanout_virtex2" 50000
}

proc SetTop {iProjectHelper synth_view} {
    SetProperty $iProjectHelper "PROP_TopDesignUnit" "Architecture|edgepenh_cw|structural" $synth_view
}

main edgepenh_cw.ise
